#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <string>
#include <exception>
#include <vector>
#include <sstream>

using namespace std;

#ifdef WIN32
#include <Winsock2.h>
#define snprintf _snprintf
#else
#define closesocket close
#define SOCKET int
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#endif

#include "igralec.h"

// -------[ socket ]------------------------------------

class TSock {
private:
	SOCKET sock;
	string buffer;
public:
	TSock();
	~TSock();

	bool Connect(char* hostname, int port);
	string readline();
	vector<string> readlineV();
	void writeline(string str);
};

TSock::TSock() : sock(-1), buffer() {}
TSock::~TSock() { closesocket(sock); }

bool TSock::Connect(char *hostname, int port) {
	sockaddr_in saddr;
	struct hostent *he;

#ifdef WIN32
	WORD wVersionRequested = MAKEWORD( 2, 2 );
	WSADATA wsaData;
	WSAStartup(wVersionRequested, &wsaData);
#endif

	he = gethostbyname(hostname);
	if (!he) return false;

	saddr.sin_family = AF_INET;
	saddr.sin_port = htons(port);
	saddr.sin_addr.s_addr = (*(struct in_addr*)he->h_addr).s_addr;
	
	sock = socket(PF_INET, SOCK_STREAM, 0);
	if (sock == -1) return false;
	if (connect(sock, (const sockaddr*)&saddr, sizeof(saddr)) != 0) return false;
	return true;
}

void chop(string &s) {
	if (s[s.length()-1] == '\n') s.erase(s.length()-1);
	if (s[s.length()-1] == '\r') s.erase(s.length()-1);
}

string TSock::readline() {
	string::size_type pos;
	while (true) {
		if ((pos=buffer.find('\n')) != string::npos) {
			string q = buffer.substr(0, pos);
			buffer.erase(0, pos+1);
			chop(q);
			return q;
		} else {
			char tmp[130];
			int s = recv(sock, tmp, 128, 0);
			if (s < 0) throw new string("Napaka pri komunikaciji s streznikom");
			if (s == 0) throw new string("Streznik je zaprl povezavo");
			tmp[s] = '\0';
			buffer += string(tmp);
		}
	}
}

vector<string> TSock::readlineV() {
	string l = readline();
	vector<string> v;
	
	string::size_type fpos, lpos = 0;
	while (lpos < l.length()) {
		fpos = l.find(' ', lpos);
		if (fpos == string::npos) fpos = l.length();
		string x = l.substr(lpos, fpos-lpos);
		v.push_back(x);
		lpos = fpos+1;
	}

	return v;
}
void TSock::writeline(string str) {
	send(sock, str.c_str(), (int)str.length(), 0);
}

// -----------------------------------------------------

void event_loop(TSock &sock) {
  vector<string> cmda;

  Igralec *t = new Igralec();
  while (true) {
    cmda = sock.readlineV();
    
    if (cmda[0] == "ADIJO") {
      delete t;
      return;
    } else if (cmda[0] == "IGRAJ") {
      int t_jaz = atoi(cmda[1].c_str());
      int t_nas = atoi(cmda[2].c_str());
      char p_jaz = cmda[3][0];
      char p_nas = cmda[4][0];

      char poteza = t->poteza(t_jaz, t_nas, p_jaz, p_nas);
      sock.writeline(string("IGRAM ") + poteza + "\n");
    } 
  }
}

void connect(char *hostname, TSock &sock) {
	if (!sock.Connect(hostname, 1180)) throw new string("Povezava s streznikom neuspesna");

	string server = sock.readline();
	printf("%s\n", server.c_str());

	sock.writeline(string("AVE ") + Igralec::Ime + "\n");
	string succ = sock.readline();

	if (succ != "PRIJAVA USPESNA") throw new string((string("Prijava na streznik neuspesna: ") + succ).c_str());
}

int main(int argc, char* argv[]) {
	char *hostname;
	TSock sock;

	if (argc < 2) hostname = "localhost";
	else hostname = argv[1];

	try {
		connect(hostname, sock);
		event_loop(sock);
	} catch (string *e) {
		printf("%s\n", e->c_str());
	}

	return 0;
}
